/**
 * 
 */
package cz.muni.fi.pb162.radio;

import java.util.Arrays;

import cz.muni.fi.pb162.music.AudioCD;
import cz.muni.fi.pb162.music.Song;
import cz.muni.fi.pb162.radio.parts.CDPlayer;
import cz.muni.fi.pb162.radio.parts.Displayable;
import cz.muni.fi.pb162.radio.parts.GivingSound;
import cz.muni.fi.pb162.radio.parts.Switchable;

/**
 * @author xpecho
 * 
 */
public class CarRadio extends UniversalRadio implements CDPlayer {

	private AudioCD cd;

	private int actualSongIndex = -1;

	/**
	 * @param radioSwitch
	 * @param speaker
	 * @param display
	 */
	public CarRadio(Switchable radioSwitch, GivingSound speaker,
			Displayable display) {
		super(radioSwitch, speaker, display);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see cz.muni.fi.pb162.radio.parts.CDPlayer#ejectCD()
	 */
	public AudioCD ejectCD() {
		AudioCD temp = cd;
		this.cd = null;
		return temp;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see cz.muni.fi.pb162.radio.parts.CDPlayer#next()
	 */
	public void next() {
		if(actualSongIndex < cd.getSongs().size() - 1){
			actualSongIndex++;
		}else {
			actualSongIndex = 0;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see cz.muni.fi.pb162.radio.parts.CDPlayer#play()
	 */
	public void play() {
		actualSongIndex = 0;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see cz.muni.fi.pb162.radio.parts.CDPlayer#previous()
	 */
	public void previous() {
		if (actualSongIndex > 0) {
			actualSongIndex--;
		}else {
			actualSongIndex = cd.getSongs().size() -1;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * cz.muni.fi.pb162.radio.parts.CDPlayer#putCD(cz.muni.fi.pb162.music.Song
	 * [])
	 */
    public void putCD(AudioCD cd) {
		this.cd = cd;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see cz.muni.fi.pb162.radio.parts.CDPlayer#stop()
	 */
	public void stop() {
		actualSongIndex = -1;
	}

	public Song actualSong() {
		return actualSongIndex > -1? cd.getSong(actualSongIndex): null;
	}

	/* (non-Javadoc)
	 * @see cz.muni.fi.pb162.radio.UniversalRadio#increaseVolume()
	 */
	public void increaseVolume() {
		// TODO Auto-generated method stub
		GivingSound speaker = getSpeaker();
		speaker.setVolume(speaker.getVolume() + 2);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + actualSongIndex;
		result = prime * result + ((cd == null) ? 0 : cd.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (!(obj instanceof CarRadio))
			return false;
		CarRadio other = (CarRadio) obj;
		if (actualSongIndex != other.actualSongIndex)
			return false;
		if (cd == null) {
			if (other.cd != null)
				return false;
		} else if (!cd.equals(other.cd))
			return false;
		return true;
	}

	
}

