package cz.muni.fi.pb162.radio;

import cz.muni.fi.pb162.radio.parts.Display;
import cz.muni.fi.pb162.radio.parts.Speaker;
import cz.muni.fi.pb162.radio.parts.Switch;

/**
 * FM radio with switch, speaker, one line 20 characters long display
 * 
 * @author �ubo� Pecho, tomorrow@mail.muni.cz .
 * @version 8.10.2009
 */
public class RadioImpl implements Radio {

    /**
     * FM frequency, on which the radio should receive signal
     */
    private double frequency = 0;
    /**
     * Switch, with which the radio can be turn on and off
     */
    private Switch radioSwitch;
    /**
     * Speaker of the radio
     */
    private Speaker speaker;
    /**
     * Display of the radio
     */
    private Display display;

    public RadioImpl() {
        radioSwitch = new Switch();
        speaker = new Speaker(30);
        display = new Display();
    }

    /**
     * Constructor that creates new radio with its all parts
     * @param maxVolume the desired maximal volume of the radio
     */
    public RadioImpl(int maxVolume) {
        radioSwitch = new Switch();
        speaker = new Speaker(maxVolume);
        display = new Display();
    }

    /**
     * Turns the radio on
     */
    public void turnOn() {
        radioSwitch.setTurnedOn(true);
    }

    /**
     * Turns the radio off
     */
    public void turnOff() {
        radioSwitch.setTurnedOn(false);
        display.setContent("");
        display.setContent("");
    }

    /**
     * Increases the volume of the radio by one point
     */
    public void increaseVolume() {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        speaker.setVolume(speaker.getVolume() + 1);
    }

    /**
     * Decreases the volume of the radio by one point
     */
    public void decreaseVolume() {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        speaker.setVolume(speaker.getVolume() - 1);
    }

    /**
     * Sets the frequency, on which the radio should receive signal. 
     * Negative frequencies are ignored
     * @param frequency
     */
    public void setFrequency(double frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0) {
            throw new IllegalArgumentException("the frequency must be greater than 0");
        }
        this.frequency = frequency;
        printFrequencyToDisplay();
    }

    /**
     * Increases the frequency by the given value
     * @param frequency the value, by which should be the frequency increased
     */
    public void increaseFrequency(double frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0) {
            throw new IllegalArgumentException("the frequency must be greater than 0");
        }
        this.frequency += frequency;
        printFrequencyToDisplay();
    }

    /**
     * Increases the frequency by the given value
     * @param frequency the value, by which should be the frequency increased
     */
    public void increaseFrequency(int frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0) {
            throw new IllegalArgumentException("the frequency must be greater than 0");
        }
        this.frequency += frequency;
        printFrequencyToDisplay();
    }

    /**
     * Decreases the frequency by the given value
     * @param frequency the value, by which should be the frequency decreased
     */
    public void decreaseFrequency(double frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0 || frequency > this.frequency) {
            throw new IllegalArgumentException("the frequency must be greater than 0" +
                    " and lower or equal than actual set frequency");
        }
        this.frequency -= frequency;
        printFrequencyToDisplay();
    }

    /**
     * Decreases the frequency by the given value
     * @param frequency the value, by which should be the frequency decreased
     */
    public void decreaseFrequency(int frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0 || frequency > this.frequency) {
            throw new IllegalArgumentException("the frequency must be greater than 0" +
                    " and lower or equal than actual set frequency");
        }
            this.frequency -= frequency;
            printFrequencyToDisplay();
    }

    /**
     * prints the frequency to the display
     */
    private void printFrequencyToDisplay() {
        display.setContent(((Double) frequency).toString());
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof RadioImpl)) return false;

        RadioImpl radio = (RadioImpl) o;

        if (Double.compare(radio.frequency, frequency) != 0) return false;
        if (display != null ? !display.equals(radio.display) : radio.display != null) return false;
        if (radioSwitch != null ? !radioSwitch.equals(radio.radioSwitch) : radio.radioSwitch != null) return false;
        if (speaker != null ? !speaker.equals(radio.speaker) : radio.speaker != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result;
        long temp;
        temp = frequency != +0.0d ? Double.doubleToLongBits(frequency) : 0L;
        result = (int) (temp ^ (temp >>> 32));
        result = 31 * result + (radioSwitch != null ? radioSwitch.hashCode() : 0);
        result = 31 * result + (speaker != null ? speaker.hashCode() : 0);
        result = 31 * result + (display != null ? display.hashCode() : 0);
        return result;
    }
}
