package cz.muni.fi.pb162.radio;

import java.util.Collections;
import java.util.Set;


/**
 * Universal remote controller to any radio
 *
 * @author �ubo� Pecho
 * @version 20.10.2009
 */
public class RemoteController {
    private Radio controlledRadio;

    private int batteryState;
    
    private Set<Radio> controlledRadios;

    public Radio getControlledRadio() {
        return controlledRadio;
    }

    public int fullyChargeBattery() {
        int difference = 100 - batteryState;
        batteryState = 100;
        return difference;
    }

    public void setControlledRadio(Radio controlledRadio) {
    	if(!controlledRadios.contains(controlledRadio)){
    		throw new IllegalArgumentException("given radio is not in cotrolled radios");
    	}
        this.controlledRadio = controlledRadio;
    }

    public void turnOnRadio() throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.turnOn();
    }

    public void turnOffRadio() throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.turnOff();
    }

    public void increaseVolume() throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.increaseVolume();
    }

    public void decreaseVolume() throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.decreaseVolume();
    }

    public void setFrequency(double frequency) throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.setFrequency(frequency);
    }

    public void increaseFrequency(double frequency) throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.increaseFrequency(frequency);
    }

    public void increaseFrequency(int frequency) throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.increaseFrequency(frequency);
    }

    public void decreaseFrequency(double frequency) throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.decreaseFrequency(frequency);
    }

    public void decreaseFrequency(int frequency) throws DischargeBatteryException {
        if (controlledRadio == null) {
            throw new IllegalArgumentException("controlled radio is not set");
        }
        if (batteryState == 0) {
            throw new DischargeBatteryException("the battery of the controller is discharged");
        }
        batteryState--;
        controlledRadio.decreaseFrequency(frequency);
    }

	/**
	 * @return the controlledRadios
	 */
	public Set<Radio> getControlledRadios() {
		return Collections.unmodifiableSet(controlledRadios);
	}

	/**
	 * @param controlledRadios the controlledRadios to set
	 */
	public void setControlledRadios(Set<Radio> controlledRadios) {
		this.controlledRadios = controlledRadios;
	}
    
    
}
